package apps.amine.bou.readerforselfoss.utils

import android.content.Context
import android.widget.Toast
import apps.amine.bou.readerforselfoss.R
import apps.amine.bou.readerforselfoss.api.selfoss.Item
import apps.amine.bou.readerforselfoss.api.selfoss.SelfossApi
import apps.amine.bou.readerforselfoss.api.selfoss.SuccessResponse
import apps.amine.bou.readerforselfoss.persistence.database.AppDatabase
import apps.amine.bou.readerforselfoss.persistence.entities.ActionEntity
import apps.amine.bou.readerforselfoss.utils.persistence.toEntity
import apps.amine.bou.readerforselfoss.utils.network.isNetworkAccessible
import retrofit2.Call
import retrofit2.Callback
import retrofit2.Response
import kotlin.concurrent.thread

/*
* Singleton class that contains the articles fetched from Selfoss, it allows sharing the items list
* between Activities and Fragments
*/
object SharedItems {
    var items: ArrayList<Item> = arrayListOf<Item>()
        get() {
            return ArrayList(field)
        }
        set(value) {
            field = ArrayList(value)
        }
    var focusedItems: ArrayList<Item> = arrayListOf<Item>()
        get() {
            return ArrayList(field)
        }
        set(value) {
            field = ArrayList(value)
        }
    var position = 0
        set(value) {
            field = when {
                value < 0 -> 0
                value > focusedItems.size -> focusedItems.size
                else -> value
            }
        }

    fun readItem(app: Context, api: SelfossApi, db: AppDatabase, item: Item) {
        if (focusedItems.contains(item)) {
            position = focusedItems.indexOf(item)
            readItemAtIndex(app, api, db)
        }
    }

    fun readItemAtIndex(app: Context, api: SelfossApi, db: AppDatabase) {
        val i = focusedItems[position]
        var tmpItems = items
        tmpItems.remove(i)
        items = tmpItems
        var tmpFocusedItems = focusedItems
        tmpFocusedItems.remove(i)
        focusedItems = tmpFocusedItems

        thread {
            db.itemsDao().delete(i.toEntity())
        }

        if (app.isNetworkAccessible(null)) {
            api.markItem(i.id).enqueue(object : Callback<SuccessResponse> {
                override fun onResponse(
                        call: Call<SuccessResponse>,
                        response: Response<SuccessResponse>
                ) {

                    //unmarkSnackbar(i, position)
                }

                override fun onFailure(call: Call<SuccessResponse>, t: Throwable) {
                    Toast.makeText(
                            app,
                            app.getString(R.string.cant_mark_read),
                            Toast.LENGTH_SHORT
                    ).show()
                    tmpItems.add(position, i)
                    tmpFocusedItems.add(position, i)
                    items = tmpItems
                    focusedItems = tmpFocusedItems

                    thread {
                        db.itemsDao().insertAllItems(i.toEntity())
                    }
                }
            })
        } else {
            thread {
                db.actionsDao().insertAllActions(ActionEntity(i.id, true, false, false, false))
            }
        }

        if (position > focusedItems.size) {
            position -= 1
        }
    }
}